@@ -29,17 +29,48 @@ open Marshallable
 let p = Clflags.progress
 let ( / ) = Filename.concat
 
-let download_sources () =
-  if !Clflags.areas = [] then raise Nothing_to_download;
+let release =
+  lazy
+    (let url =
+       sprintf "%s/dists/%s/Release" !Clflags.mirror_sources !Clflags.suite
+     in
+     ("curl", [| "curl"; "--silent"; url |])
+     |> Lwt_process.pread |> Lwt_main.run |> Utils.parse_release)
+
+let automatic_components =
+  lazy
+    (let release = Lazy.force release in
+     let r =
+       match Ben.Stanza.find "components" release with
+       | x -> String.split_on_char ' ' x
+       | exception Not_found -> []
+     in
+     p "Automatically detected components: %s\n" (String.concat " " r);
+     r)
+
+let automatic_architectures =
+  lazy
+    (let release = Lazy.force release in
+     let r =
+       match Ben.Stanza.find "architectures" release with
+       | x -> String.split_on_char ' ' x
+       | exception Not_found -> []
+     in
+     let r = List.filter (fun x -> x <> "all") r in
+     p "Automatically detected architectures: %s\n" (String.concat " " r);
+     r)
+
+let download_sources components =
+  if components = [] then raise Nothing_to_download;
   let wquiet = if !Clflags.verbose then "" else "--silent" in
   let dst = !Clflags.cache_dir / "Sources" in
   let tmp = Filename.temp_file "Sources." "" in
   let commands =
     Ben.Parallel.map
-      (fun area ->
+      (fun component ->
         let url =
           sprintf "%s/dists/%s/%s/source/Sources%s" !Clflags.mirror_sources
-            !Clflags.suite area
+            !Clflags.suite component
             (Ben.Compression.extension !Clflags.preferred_compression_format)
         in
         if !Clflags.dry_run then p "Would download %s\n" url;
@@ -49,7 +80,7 @@ let download_sources () =
             tmp
         in
         cmd)
-      !Clflags.areas
+      components
   in
   let cmd = sprintf "%s && mv %s %s" (String.concat " && " commands) tmp dst in
   if not !Clflags.dry_run then (
@@ -58,17 +89,17 @@ let download_sources () =
     if r <> 0 then raise (Curl_error r)
     else FileUtil.rm ~force:FileUtil.Force [ tmp ])
 
-let download_binaries arch =
-  if !Clflags.areas = [] then raise Nothing_to_download;
+let download_binaries components arch =
+  if components = [] then raise Nothing_to_download;
   let wquiet = if !Clflags.verbose then "" else "--silent" in
   let dst = (!Clflags.cache_dir / "Packages_") ^ arch in
   let tmp = Filename.temp_file "Packages." "" in
   let commands =
     Ben.Parallel.map
-      (fun area ->
+      (fun component ->
         let url =
           sprintf "%s/dists/%s/%s/binary-%s/Packages%s" !Clflags.mirror_binaries
-            !Clflags.suite area arch
+            !Clflags.suite component arch
             (Ben.Compression.extension !Clflags.preferred_compression_format)
         in
         if !Clflags.dry_run then p "Would download %s\n" url;
@@ -78,7 +109,7 @@ let download_binaries arch =
             tmp
         in
         cmd)
-      !Clflags.areas
+      components
   in
   let cmd = sprintf "%s && mv %s %s" (String.concat " && " commands) tmp dst in
   if not !Clflags.dry_run then (
@@ -88,11 +119,15 @@ let download_binaries arch =
     else FileUtil.rm ~force:FileUtil.Force [ tmp ])
 
 let download_all architectures =
-  download_sources ();
-  Ben.Parallel.iter download_binaries architectures
+  let components =
+    match !Clflags.components with
+    | None -> Lazy.force automatic_components
+    | Some x -> x
+  in
+  download_sources components;
+  Ben.Parallel.iter (download_binaries components) architectures
 
-let save_cache () =
-  let architectures = !Clflags.debian_architectures in
+let save_cache architectures =
   if !Clflags.use_cache then
     let file = Clflags.get_cache_file () in
     let () = p "Generating %s...\n" file in
@@ -106,8 +141,13 @@ let save_cache () =
     Marshal.dump file data
 
 let main () =
-  download_all !Clflags.debian_architectures;
-  save_cache ()
+  let architectures =
+    match !Clflags.debian_architectures with
+    | None -> Lazy.force automatic_architectures
+    | Some x -> x
+  in
+  download_all architectures;
+  save_cache architectures
 
 let frontend =
   {
@@ -116,3 +156,12 @@ let frontend =
     Frontend.anon_fun = (fun _ -> ());
     Frontend.help = [];
   }
+
+let archs_list () =
+  let architectures =
+    match !Clflags.debian_architectures with
+    | None -> Lazy.force automatic_architectures
+    | Some x -> x
+  in
+  architectures @ !Clflags.ignored_architectures
+  |> uniq |> List.sort Stdlib.compare
