@@ -725,7 +725,7 @@ let main _ =
   close ()
 
 let anon_fun file =
-  if Core.ends_with file ".ben" then input_source := Types.File file
+  if String.ends_with file ~suffix:".ben" then input_source := Types.File file
 
 let frontend =
   {
