@@ -47,11 +47,13 @@ let read_global_config file =
       (fun key value ->
         match (key, value) with
         | "architectures", archs ->
-            Clflags.debian_architectures := to_string_l "architectures" archs
+            Clflags.debian_architectures :=
+              to_string_l_auto "architectures" archs
         | "ignored", archs ->
             Clflags.ignored_architectures := to_string_l "ignored" archs
         | "suite", EString suite -> Clflags.suite := suite
-        | "areas", areas -> Clflags.areas := to_string_l "areas" areas
+        | ("areas" | "components"), components ->
+            Clflags.components := to_string_l_auto "components" components
         | "base", EString path -> base := path
         | "config-dir", EString path -> config_dir := path
         | "cache-dir", EString dir -> Clflags.cache_dir := dir
@@ -469,7 +471,7 @@ let main () =
           [] transitions
       in
       (* Read ben.cache *)
-      let cache = read_cache (Clflags.archs_list ()) in
+      let cache = read_cache (Ben_download.archs_list ()) in
       (* Emit warnings for missing source packages *)
       let () =
         if !Clflags.verbose || not !Clflags.quiet then
