@@ -78,3 +78,11 @@ let dump_xhtml_to_file filename xhtml =
   let xhtml = Format.asprintf "%a" (Tyxml.Html.pp ()) xhtml in
   let () = output_string outchan xhtml in
   close_out outchan
+
+let parse_release string =
+  let stanzas =
+    Lexer.stanza_fold (fun p accu -> p :: accu) (Lexing.from_string string) []
+  in
+  match stanzas with
+  | [ s ] -> s
+  | _ -> failwith "not exactly 1 stanza in Release"
